<?php

/**
 * @file
 * Provides Wordbee Beebox plugin controller.
 */

require_once(drupal_get_path('module','tmgmt_wordbee').'/inc/BeeboxAPI.php');
require_once(drupal_get_path('module','tmgmt_wordbee').'/inc/TMGMTCustomFileformatXLIFF.class.php');

/**
 * Wordbee Beebox translator plugin controller.
 */
class TMGMTWordbeeBeeboxPluginController extends TMGMTDefaultTranslatorPluginController {

    /**
     * API_Calls instance
     *
     * @var API_Calls
     */
    private $apiCalls;
	
	/**
	 * TMGMT 7.x-1.0-rc1 sounds to call "isAvailable" several time. It can be a problem
	 *
	 * @var isAvailableCache
	 */
	private $isAvailableCache;
	
    /**
     * Overrides TMGMTDefaultTranslatorPluginController::isAvailable().
	 * 
	 * TMGMT 7.x-1.0-rc1 bug : Page "en/admin/config/regional/tmgmt_translator" use the same "isAvailable" status for all instance of the Beebox Connector.
     */
    public function isAvailable(TMGMTTranslator $translator) {
        if($this->prepareApiCalls($translator)) {
			if(isset($this->isAvailableCache)) // Because TMGMT 7.x-1.0-rc1 call "isAvailable" few times, we only handle the first request.
				return $this->isAvailableCache;
		
			try {
				$connected = $this->apiCalls->connect();
			} catch (Exception $e) {
				$trace = $e->getTrace();

				drupal_set_message(t('An error occured with @name : @message', array('@name' => $translator->label, '@message' => $e->getMessage())), 'error'); 
				$connected = FALSE;
			}
			$this->apiCalls->disconnect();
		} else
			$connected = FALSE;
		
		$this->isAvailableCache = $connected; // Cache
        return $connected;
    }

    /**
     * Overrides TMGMTDefaultTranslatorPluginController::canTranslate().
     */
    public function canTranslate(TMGMTTranslator $translator, TMGMTJob $job){
        if (!parent::canTranslate($translator, $job))
            return FALSE;

        $this->prepareApiCalls($translator);
        try{
            $languages = $this->apiCalls->getProjectLanguages();
            $this->apiCalls->disconnect();
			
			if(!isset($languages[$translator->mapToRemoteLanguage(($job->source_language))])){
				return false;
			}
			elseif(!array_key_exists($translator->mapToRemoteLanguage($job->target_language), $languages[$translator->mapToRemoteLanguage(($job->source_language))])){
				return false;
			}
        } catch (Exception $e) {
            return FALSE;
        }
        return TRUE;
    }
    
    /**
     * Overrides TMGMTDefaultTranslatorPluginController::requestTranslation).
     */
    public function requestTranslation(TMGMTJob $job) {
        if($job->getTranslator()->getSetting('leave_xliff_target_empty'))
            $fileformat = new TMGMTCustomFileformatXLIFF();
        else
            $fileformat = new TMGMTFileformatXLIFF();

        $filename = $job->tjid . '-drupal_connector.xliff';
        $xliff_file = $fileformat->export($job);

        $translator = $job->getTranslator();
        $this->prepareApiCalls($translator);

        try {
            $this->apiCalls->sendFile($xliff_file, $filename, $translator->mapToRemoteLanguage($job->source_language));
            $this->apiCalls->sendFile('{"locales":["'.$translator->mapToRemoteLanguage($job->target_language).'"]}', $filename.'.beebox', $translator->mapToRemoteLanguage($job->source_language));
            /*if($this->apiCalls->scanRequired())
                $this->apiCalls->scanFiles();*/
			$job->submitted('Job has been submitted to Beebox.');
        } catch (Exception $e) {
            watchdog_exception('tmgmt_wordbee', $e);
            $job->rejected('Job has been rejected with following error: @error', array('@error' => $e->getMessage()), 'error');
        }
        $this->apiCalls->disconnect();
    }
    
    /**
     * Overrides TMGMTDefaultTranslatorPluginController::abortTranslation().
     */
    public function abortTranslation(TMGMTJob $job){
        $filename = $job->tjid.'-drupal_connector.xliff';
		$translator = $job->getTranslator();
        $this->prepareApiCalls($translator);
        try{
            $workprogress = $this->apiCalls->getWorkprogress(array($filename));
            if(count($workprogress) > 0){
                $this->apiCalls->deleteFile($filename, $translator->mapToRemoteLanguage($job->source_language));
                $this->apiCalls->deleteFile($filename.'.beebox', $translator->mapToRemoteLanguage($job->source_language));
                $job->aborted('Job removed');
                return true;
            }
        } catch(Exception $e){
            watchdog_exception('tmgmt_wordbee', $e);
            $job->rejected('Job has not been cancelled with following error: @error', array('@error' => $e->getMessage()), 'error');
        }
		$this->apiCalls->disconnect();
        
        return false;
    }
	
	/**
	 * TMGMT-7.x-1.0-rc1 compatibility
	 */
	public function cancelTranslation(TMGMTJob $job){
		return $this->abortTranslation($job);
	}
    
    /**
     * Checks if a job is complete on the Beebox, if it's the case, the job is downloaded and set to 'finished' in tmgmt
     * 
     * @param TMGMTJob $job
     */
    public function updateCompletedJob(TMGMTJob $job){
        $this->prepareApiCalls($job->getTranslator());
        $content = $this->apiCalls->getWorkprogress(array($job->tjid.'-drupal_connector.xliff'));
		$workprogress = json_decode($content);
        $fileformat = new TMGMTCustomFileformatXLIFF();
		
        foreach ($workprogress->files as $work) {
            if ($work->uptodate) {
                $file = $this->apiCalls->getFile($work->file, $work->locale);
                if($fileformat->validateImport($file)){
                    $job->addTranslatedData($fileformat->import($file));
                    $job->finished('Translation done');
                }
            }
        }
        $this->apiCalls->disconnect();
    }

    /**
     * Instanciate 'API_calls' if needed, this method should be called before using $this->apiCalls
     * 
     * @param TMGMTTranslator $translator
     */
    private function prepareApiCalls(TMGMTTranslator $translator) {
		// 1. Check if we got all settings.
		if(!$translator->getSetting('url') || !$translator->getSetting('projectKey') || !$translator->getSetting('username') || !$translator->getSetting('password'))
            return false;
		
		// 2. Connect
		$module_infos = system_get_info('module', 'tmgmt_wordbee');
        if (!isset($this->apiCalls))
            $this->apiCalls = new BeeboxAPI(
				'WB-Drupal',
				$module_infos['version'],
				$translator->getSetting('url'),
				$translator->getSetting('projectKey'),
				$translator->getSetting('username'),
				$translator->getSetting('password')
			);
			
		return true;
    }
}
